// tempus 2000.11
#ifndef TUTA_SYSTEM_CLASS
#define TUTA_SYSTEM_CLASS
//
// (c) Copyright 1995-2000, MZA Associates Corporation
//

#include "tempus.h"


#include "VacuumProp.h"
#include "Telescope.h"
#include "TargetBoard.h"
#include "UniformWave.h"

class TutA : public System
{
public:

   // Parameters
   float target_range;
   float tel_focus_range;
   float tel_diam;
   float hel_wavelength;
   int tbd_nxy;
   float tbd_dxy;

   // Subsystems
   VacuumProp vacuumprop;

   Telescope telescope;

   TargetBoard targetboard;

   UniformWave uw;


   // Inputs

   // Outputs

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< WaveTrain >  vacuumprop_incomingIncident;
   Output< Recallable<float> >  vacuumprop_propagationDistance;
   Output< WaveTrain >  telescope_incomingIncident;
   Output< bool >  targetboard_on;
   Output< double >  targetboard_exposureInterval;
   Output< double >  targetboard_exposureLength;
   Output< double >  targetboard_sampleInterval;

public:
   TutA(SystemNode *parent, char *name,
        float _target_range,
        float _tel_focus_range,
        float _tel_diam,
        float _hel_wavelength,
        int _tbd_nxy,
        float _tbd_dxy) : 
      System(parent, name),
      target_range(_target_range),
      tel_focus_range(_tel_focus_range),
      tel_diam(_tel_diam),
      hel_wavelength(_hel_wavelength),
      tbd_nxy(_tbd_nxy),
      tbd_dxy(_tbd_dxy),
      vacuumprop(this, "vacuumprop"),
      telescope(this, "telescope",
         tel_focus_range,
         tel_diam/2.0,
         0.0),
      targetboard(this, "targetboard",
         hel_wavelength,
         tbd_nxy,
         tbd_dxy),
      uw(this, "uw",
         hel_wavelength,
         hel_wavelength,
         1.0),
      vacuumprop_incomingIncident(this, "vacuumprop_incomingIncident"),
      vacuumprop_propagationDistance(this, "vacuumprop_propagationDistance"),
      telescope_incomingIncident(this, "telescope_incomingIncident"),
      targetboard_on(this, "targetboard_on"),
      targetboard_exposureInterval(this, "targetboard_exposureInterval"),
      targetboard_exposureLength(this, "targetboard_exposureLength"),
      targetboard_sampleInterval(this, "targetboard_sampleInterval")
   {
      targetboard.incident <<= vacuumprop.outgoingTransmitted;
      telescope.outgoingIncident <<= uw.transmitted;
      vacuumprop.outgoingIncident <<= telescope.outgoingTransmitted;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      vacuumprop.incomingIncident <<= vacuumprop_incomingIncident;
      vacuumprop.propagationDistance <<= vacuumprop_propagationDistance;
      telescope.incomingIncident <<= telescope_incomingIncident;
      targetboard.on <<= targetboard_on;
      targetboard.exposureInterval <<= targetboard_exposureInterval;
      targetboard.exposureLength <<= targetboard_exposureLength;
      targetboard.sampleInterval <<= targetboard_sampleInterval;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase & /* input */);
   void respondToOutputRequest (const OutputBase * output)
   {
      double t = now();
      if(*output == vacuumprop_incomingIncident)
      {
         vacuumprop_incomingIncident = (WaveTrain)(WaveTrain());
      }
      else if(*output == vacuumprop_propagationDistance)
      {
         vacuumprop_propagationDistance = (Recallable<float>)(recallableFloat(target_range));
      }
      else if(*output == telescope_incomingIncident)
      {
         telescope_incomingIncident = (WaveTrain)(WaveTrain());
      }
      else if(*output == targetboard_on)
      {
         targetboard_on = (bool)(true);
      }
      else if(*output == targetboard_exposureInterval)
      {
         targetboard_exposureInterval = (double)(1.0e-3);
      }
      else if(*output == targetboard_exposureLength)
      {
         targetboard_exposureLength = (double)(1.0e-6);
      }
      else if(*output == targetboard_sampleInterval)
      {
         targetboard_sampleInterval = (double)(0.0);
      }
   }
};

#endif // TUTA_SYSTEM_CLASS
